#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion
// Code provided by 'total3', compiled and posted by 'Elliott Wave'.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    [Description("From the July issue of TASC.")]
    public class LeaderOfMACD : Indicator
    {
        #region Variables
        // Wizard generated variables
        // User defined variables (add any user defined variables below)
		private int fast = 12;
		private int slow = 26;
		private int smooth = 9;
		
		private DataSeries fastMACD;
		private DataSeries macdAvg2;
		private DataSeries fastEma;
		private DataSeries slowEma;
		private DataSeries slow1S;
		private DataSeries slowF;
		private DataSeries fast1S;
		private DataSeries fastF;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.Green, "Macd"));
			Add(new Plot(Color.DarkViolet, "Avg"));
			Add(new Plot(new Pen(Color.Navy, 2), PlotStyle.Bar, "Diff"));
			Add(new Plot(Color.Red, "Leader"));
			
			Add(new Line(Color.DarkGray, 0, "Zero line"));
			
			fastMACD = new DataSeries(this);
			macdAvg2 = new DataSeries(this);
			
			fastEma = new DataSeries(this);
			slowEma = new DataSeries(this);
			
			slow1S = new DataSeries(this);
			slowF = new DataSeries(this);
			
			fast1S = new DataSeries(this);
			fastF = new DataSeries(this);
			
			CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            // Use this method for calculating your indicator values. Assign a value to each
            // plot below by replacing 'Close[0]' with your own formula.
			
			if (CurrentBar == 0)
			{
			
			fastMACD.Set(Input[0]);
			macdAvg2.Set(Input[0]);
			
			fastEma.Set(Input[0]);
			slowEma.Set(Input[0]);
			
			slow1S.Set(Input[0]);
			slowF.Set(Input[0]);
			
			fast1S.Set(Input[0]);
			fastF.Set(Input[0]);
			
			Value.Set(0);
			Avg.Set(0);
			Diff.Set(0);
			Leader.Set(0);
			}
			else
			{
			
			slowEma.Set(Close[0] - EMA(Input,Slow)[0]) ;
			slow1S.Set(EMA(slowEma,Slow)[0]);
			slowF.Set(slow1S[0] + EMA(Input,Slow)[0]);
			double slowFX = slowF[0];
			
			fastEma.Set(Close[0] - EMA(Input,Fast)[0]) ;
			fast1S.Set(EMA(fastEma,Fast)[0]);
			fastF.Set(fast1S[0] + EMA(Input,Fast)[0]);
			double fastFX = fastF[0];
			
			
			
			fastMACD.Set((EMA(Input,Fast)[0]) - (EMA(Input,Slow)[0]));
			double macd = fastMACD[0] ;
			
			macdAvg2.Set(EMA(fastMACD,Smooth)[0]);
			double macdAvg = macdAvg2[0];
			
			
			Value.Set(macd);
			Avg.Set(macdAvg);
			Diff.Set(macd - macdAvg);
			
			Leader.Set(fastFX - slowFX);
			}
        }

        #region Properties
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Avg
		{
		get { return Values[1]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Default
		{
		get { return Values[0]; }
		}
		
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Diff
		{
		get { return Values[2]; }
		}
		
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Leader
		{
		get { return Values[3]; }
		}
		
		
		/// <summary>
		/// </summary>
		[Description("Number of bars for fast EMA")]
		[Category("Parameters")]
		public int Fast
		{
		get { return fast; }
		set { fast = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Number of bars for slow EMA")]
		[Category("Parameters")]
		public int Slow
		{
		get { return slow; }
		set { slow = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing")]
		[Category("Parameters")]
		public int Smooth
		{
		get { return smooth; }
		set { smooth = Math.Max(1, value); }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private LeaderOfMACD[] cacheLeaderOfMACD = null;

        private static LeaderOfMACD checkLeaderOfMACD = new LeaderOfMACD();

        /// <summary>
        /// From the July issue of TASC.
        /// </summary>
        /// <returns></returns>
        public LeaderOfMACD LeaderOfMACD(int fast, int slow, int smooth)
        {
            return LeaderOfMACD(Input, fast, slow, smooth);
        }

        /// <summary>
        /// From the July issue of TASC.
        /// </summary>
        /// <returns></returns>
        public LeaderOfMACD LeaderOfMACD(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            checkLeaderOfMACD.Fast = fast;
            fast = checkLeaderOfMACD.Fast;
            checkLeaderOfMACD.Slow = slow;
            slow = checkLeaderOfMACD.Slow;
            checkLeaderOfMACD.Smooth = smooth;
            smooth = checkLeaderOfMACD.Smooth;

            if (cacheLeaderOfMACD != null)
                for (int idx = 0; idx < cacheLeaderOfMACD.Length; idx++)
                    if (cacheLeaderOfMACD[idx].Fast == fast && cacheLeaderOfMACD[idx].Slow == slow && cacheLeaderOfMACD[idx].Smooth == smooth && cacheLeaderOfMACD[idx].EqualsInput(input))
                        return cacheLeaderOfMACD[idx];

            LeaderOfMACD indicator = new LeaderOfMACD();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Fast = fast;
            indicator.Slow = slow;
            indicator.Smooth = smooth;
            indicator.SetUp();

            LeaderOfMACD[] tmp = new LeaderOfMACD[cacheLeaderOfMACD == null ? 1 : cacheLeaderOfMACD.Length + 1];
            if (cacheLeaderOfMACD != null)
                cacheLeaderOfMACD.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheLeaderOfMACD = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// From the July issue of TASC.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.LeaderOfMACD LeaderOfMACD(int fast, int slow, int smooth)
        {
            return _indicator.LeaderOfMACD(Input, fast, slow, smooth);
        }

        /// <summary>
        /// From the July issue of TASC.
        /// </summary>
        /// <returns></returns>
        public Indicator.LeaderOfMACD LeaderOfMACD(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            return _indicator.LeaderOfMACD(input, fast, slow, smooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// From the July issue of TASC.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.LeaderOfMACD LeaderOfMACD(int fast, int slow, int smooth)
        {
            return _indicator.LeaderOfMACD(Input, fast, slow, smooth);
        }

        /// <summary>
        /// From the July issue of TASC.
        /// </summary>
        /// <returns></returns>
        public Indicator.LeaderOfMACD LeaderOfMACD(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.LeaderOfMACD(input, fast, slow, smooth);
        }

    }
}
#endregion
